from __future__ import unicode_literals

import datetime

try:
    from urllib.parse import urljoin
except ImportError:  # For Python 2
    from urlparse import urljoin  # noqa

import requests

from .exceptions import FixerioException

BASE_URL = 'http://data.fixer.io/api/'

LATEST_PATH = 'latest'


class Fixerio(object):
    """ A client for Fixer.io. """

    def __init__(self, access_key, symbols=None):
        """
        :param access_key: your API Key.
        :type access_key: str or unicode
        :param symbols: currency symbols to request specific exchange rates.
        :type symbols: list or tuple
        """
        self.access_key = access_key
        self.symbols = symbols

    def _create_payload(self, symbols):
        """ Creates a payload with no none values.

        :param symbols: currency symbols to request specific exchange rates.
        :type symbols: list or tuple
        :return: a payload.
        :rtype: dict
        """
        payload = {'access_key': self.access_key}
        if symbols is not None:
            payload['symbols'] = ','.join(symbols)

        return payload

    def latest(self, symbols=None):
        """ Get the latest foreign exchange reference rates.

        :param symbols: currency symbols to request specific exchange rates.
        :type symbols: list or tuple
        :return: the latest foreign exchange reference rates.
        :rtype: dict
        :raises FixerioException: if any error making a request.
        """
        try:
            symbols = symbols or self.symbols
            payload = self._create_payload(symbols)

            url = BASE_URL + LATEST_PATH

            response = requests.get(url, params=payload)

            response.raise_for_status()

            return response.json()
        except requests.exceptions.RequestException as ex:
            raise FixerioException(str(ex))

    def historical_rates(self, date, symbols=None):
        """
        Get historical rates for any day since `date`.

        :param date: a date
        :type date: date or str
        :param symbols: currency symbols to request specific exchange rates.
        :type symbols: list or tuple
        :return: the historical rates for any day since `date`.
        :rtype: dict
        :raises FixerioException: if any error making a request.
        """
        try:
            if isinstance(date, datetime.date):
                # Convert date to ISO 8601 format.
                date = date.isoformat()

            symbols = symbols or self.symbols
            payload = self._create_payload(symbols)

            url = BASE_URL + date

            response = requests.get(url, params=payload)

            response.raise_for_status()

            return response.json()
        except requests.exceptions.RequestException as ex:
            raise FixerioException(str(ex))



from __future__ import unicode_literals

import unittest

from fixerio.client import Fixerio


class FixerioInitTestCase(unittest.TestCase):
    def test_raises_if_access_key_is_not_passed(self):
        with self.assertRaises(TypeError):
            Fixerio()

    def test_sets_access_key(self):
        access_key = 'test-access-key'

        client = Fixerio(access_key)

        self.assertEqual(client.access_key, access_key)

    def test_sets_none_symbols_attribute_if_it_is_not_passed(self):
        client = Fixerio('test-access-key')

        self.assertIsNone(client.symbols)

    def test_sets_symbols_attribute(self):
        symbols = ['USD', 'GBP']

        client = Fixerio('test-access-key', symbols=symbols)

        self.assertEqual(client.symbols, symbols)



from __future__ import unicode_literals

import json
import unittest
from datetime import date

try:
    from urllib.parse import urljoin  # noqa: F401
    from urllib.parse import urlencode  # noqa: F401
except ImportError:  # For Python 2
    from urlparse import urljoin  # noqa: F401
    from urllib import urlencode  # noqa: F401

import responses

from fixerio.client import Fixerio
from fixerio.exceptions import FixerioException

BASE_URL = 'http://data.fixer.io/api'


class FixerioHistoricalRatesTestCase(unittest.TestCase):
    def setUp(self):
        self.access_key = 'test-access-key'
        self.date = date(2000, 1, 3)
        self.path = '/{0}'.format(self.date.isoformat())
        query = urlencode({'access_key': self.access_key})
        self.url = BASE_URL + self.path + '?' + query

    @responses.activate
    def test_returns_historical_rates(self):
        expected_response = {'base': 'EUR',
                             'date': '2000-01-03',
                             'rates': {'GBP': 0.6246}}
        responses.add(responses.GET,
                      self.url,
                      body=json.dumps(expected_response),
                      content_type='application/json')
        client = Fixerio(self.access_key)
        response = client.historical_rates(date=self.date.isoformat())

        self.assertDictEqual(response, expected_response)
        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(request.url, self.url)
        self.assertIsNone(request.body)

        client = Fixerio(self.access_key)
        response = client.historical_rates(date=self.date)

        self.assertDictEqual(response, expected_response)
        request = responses.calls[1].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(request.url, self.url)
        self.assertIsNone(request.body)

    @responses.activate
    def test_raises_exception_if_bad_request(self):
        responses.add(responses.GET,
                      self.url,
                      body="{'success': false}",
                      status=400,
                      content_type='text/json')

        with self.assertRaises(FixerioException)as ex:
            client = Fixerio(self.access_key)
            client.historical_rates(date=self.date)

        expected_message = (('400 Client Error: Bad Request for url: '
                             '{0}').format(self.url))
        self.assertEqual(str(ex.exception), expected_message)


class FixerioHistoricalRatesSymbolsTestCase(unittest.TestCase):
    def setUp(self):
        self.access_key = 'test-access-key'
        self.date = date(2000, 1, 3)
        self.path = '/{0}'.format(self.date.isoformat())
        query = urlencode({'access_key': self.access_key})
        self.url = BASE_URL + self.path + '?' + query

    @responses.activate
    def test_returns_historical_rates_for_symbols_passed_in_constructor(self):
        symbols = ['USD', 'GBP']
        expected_response = {"base": "EUR",
                             "date": "2000-01-03",
                             "rates": {"GBP": 0.6246, "USD": 1.009}}
        responses.add(responses.GET,
                      self.url,
                      body=json.dumps(expected_response),
                      content_type='application/json')

        client = Fixerio(self.access_key, symbols=symbols)
        response = client.historical_rates(date=self.date)

        self.assertDictEqual(response, expected_response)
        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        symbols_str = ','.join(symbols)
        params = urlencode(
            {'access_key': self.access_key, 'symbols': symbols_str})
        expected_path = '{url}?{params}'.format(url=self.path, params=params)
        expected_url = BASE_URL + expected_path
        self.assertEqual(request.url, expected_url)
        self.assertIsNone(request.body)

    @responses.activate
    def test_returns_historical_rates_for_symbols_passed_in_method(self):
        symbols = ['USD', 'GBP']
        expected_response = {"base": "EUR",
                             "date": "2000-01-03",
                             "rates": {"GBP": 0.6246, "USD": 1.009}}
        responses.add(responses.GET,
                      self.url,
                      body=json.dumps(expected_response),
                      content_type='application/json')

        client = Fixerio(self.access_key)
        response = client.historical_rates(date=self.date, symbols=symbols)

        self.assertDictEqual(response, expected_response)
        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        symbols_str = ','.join(symbols)
        params = urlencode(
            {'access_key': self.access_key, 'symbols': symbols_str})
        expected_path = '{url}?{params}'.format(url=self.path, params=params)
        expected_url = BASE_URL + expected_path
        self.assertEqual(request.url, expected_url)
        self.assertIsNone(request.body)

    @responses.activate
    def test_returns_historical_rates_for_symbols_passed_if_both(self):
        symbols = ['USD', 'GBP']
        other_symbols = ['JPY', 'EUR']
        expected_response = {"base": "EUR",
                             "date": "2000-01-03",
                             "rates": {"GBP": 0.6246, "USD": 1.009}}
        responses.add(responses.GET,
                      self.url,
                      body=json.dumps(expected_response),
                      content_type='application/json')

        client = Fixerio(self.access_key, symbols=other_symbols)
        response = client.historical_rates(date=self.date, symbols=symbols)

        self.assertDictEqual(response, expected_response)
        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        symbols_str = ','.join(symbols)
        params = urlencode(
            {'access_key': self.access_key, 'symbols': symbols_str})
        expected_path = '{url}?{params}'.format(url=self.path, params=params)
        expected_url = BASE_URL + expected_path
        self.assertEqual(request.url, expected_url)
        self.assertIsNone(request.body)



import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('fixerio/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

with open('README.rst') as f:
    readme = f.read()
with open('CHANGELOG.rst') as f:
    changelog = f.read()

requirements = ['requests>=2.0']

setup(
    name='fixerio',
    version=version,
    description='A Python client for Fixer.io',
    long_description=readme + '\n\n' + changelog,
    author="Adrian Matellanes",
    author_email='matellanesadrian@gmail.com',
    url='https://github.com/amatellanes/fixerio',
    install_requires=requirements,
    license='MIT License',
    packages=['fixerio'],
    package_dir={'fixerio': 'fixerio'},
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)
